package com.jasamedika.medifirst2000.service;

import java.util.Map;

import org.springframework.data.jpa.domain.Specification;

import com.jasamedika.medifirst2000.entities.RegistrasiAset;
import com.jasamedika.medifirst2000.vo.DistribusiBarangInvestasiVO;
import com.jasamedika.medifirst2000.vo.HistoryAlatVO;
import com.jasamedika.medifirst2000.vo.RegistrasiAsetVO;

/**
 * @author adik
 */
public interface RegistrasiAsetService <T> extends BaseVoService<RegistrasiAset, RegistrasiAsetVO, String> {

	Map<String, Object> saveDistribusiBarang(DistribusiBarangInvestasiVO vo);

	Map<String, Object> detailAset(String noRec);

	Map<String, Object> updateAset(RegistrasiAsetVO vo);

	Map<String, Object> gridListAset(Integer page, Integer limit, String sort, String dir, String periodeAwal,
			String periodeAhir, Integer ruanganId);

	Map<String, Object> saveDokumen(HistoryAlatVO historyAlatVO);

	Map<String, Object> getFormatDownload(Integer noRec);

	Map<String, Object> daftarDokumen(String noRec, String kategori);
	
	Map<String, Object> daftarDokumen(String noRec);


}



